/*--------------------------------------------------------------*\
		module	:	MySocket.c
		purpose :	ȒPt@C] \Pbg֌W
		author	:	_@ Kengo Jinno
		create	:	2011/02/25
		update	:	2011/04/08
\*--------------------------------------------------------------*/

#include	"PCH.h"
#include	"MySocket.h"
#include	"resource.h"
#include	"Tools.h"

#define	MSG_MAGIC	0x11030201

static	BOOL	RecvS2cMsg( SOCKET hSocket, NpMyServer2ClientMsg p );

BOOL	InitMySocket( HWND hWnd, NpMySocketInfo pInfo )
{
	pInfo->bListen = FALSE;
	pInfo->hSocket = INVALID_SOCKET;
	pInfo->hSockClient = INVALID_SOCKET;
	pInfo->szSendTo[0] = _T('\0');
	pInfo->uPort = 0;
	pInfo->bCleanUp = FALSE;
	pInfo->uAsyncMsg = RegisterWindowMessage( _T("MySftSocketMessage") );
	
	pInfo->hMsgBuff = NULL;
	pInfo->dwBuffLen = 0;
	pInfo->dwRecvLen = 0;
	
	return TRUE;
}

BOOL	StartMySocket( HWND hWnd, NpMySocketInfo pInfo )
{
	WSADATA	wsadata;
	pInfo->bCleanUp = ( WSAStartup( MAKEWORD(1, 1), &wsadata ) == 0 );
	return pInfo->bCleanUp;
}

BOOL	QuitMySocket( HWND hWnd, NpMySocketInfo pInfo )
{
	if( pInfo->bCleanUp ) {
		WSACleanup();
		pInfo->bCleanUp = FALSE;
	}
	if( pInfo->hMsgBuff != NULL ) {
		GlobalFree( pInfo->hMsgBuff );
		pInfo->hMsgBuff = NULL;
	}
	pInfo->dwBuffLen = 0;
	pInfo->dwRecvLen = 0;
	return TRUE;
}

BOOL	OpenMySocket( HWND hWnd, NpMySocketInfo pInfo )
{
	BOOL	ret = TRUE;
	
	if( !IsOpenedMySocket( pInfo ) ) {
		/*	Socket	*/
		pInfo->hSocket = socket( PF_INET, SOCK_STREAM, 0 );
		if( pInfo->hSocket == INVALID_SOCKET ) {
			pInfo->uErrorID = IDS_ERROR_SOCKET;
			ret = FALSE;
		}
		if( pInfo->bListen ) {
			/*	Bind	*/
			if( ret ) {
				SOCKADDR_IN	sockAddr;
				ZeroMemory( &sockAddr, sizeof(sockAddr) );
				sockAddr.sin_family = AF_INET;
				sockAddr.sin_addr.s_addr = htonl( INADDR_ANY );
				sockAddr.sin_port = htons( (u_short)pInfo->uPort );
				if( bind( pInfo->hSocket, (SOCKADDR*)&sockAddr, sizeof(sockAddr) ) == SOCKET_ERROR ) {
					pInfo->uErrorID = IDS_ERROR_BIND;
					ret = FALSE;
				}
			}
			/*	Listen	*/
			if( ret ) {
				if( listen( pInfo->hSocket, 5 ) == SOCKET_ERROR ) {
					pInfo->uErrorID = IDS_ERROR_LISTEN;
					ret = FALSE;
				}
			}
		} else {
			/*	Connect	*/
			if( ret ) {
				SOCKADDR_IN	sockAddr;
				ZeroMemory( &sockAddr, sizeof(sockAddr) );
				sockAddr.sin_family = AF_INET;
				sockAddr.sin_addr.s_addr = inet_addr( pInfo->szSendTo );
				if( sockAddr.sin_addr.s_addr == INADDR_NONE ) {
					LPHOSTENT	lphost;
					lphost = gethostbyname( pInfo->szSendTo );
					if( lphost != NULL ) {
						sockAddr.sin_addr.s_addr = ((LPIN_ADDR)lphost->h_addr)->s_addr;
					}
				}
				sockAddr.sin_port = htons( (u_short)pInfo->uPort );
				if( sockAddr.sin_addr.s_addr == INADDR_NONE ||
					connect( pInfo->hSocket, (SOCKADDR*)&sockAddr, sizeof(sockAddr) ) == SOCKET_ERROR ) {
					pInfo->uErrorID = IDS_ERROR_CONNECT;
					ret = FALSE;
				}
			}
		}
		/*	WSAAsyncSelect	*/
		if( ret ) {
			long	lEvent = FD_CLOSE|FD_READ;
			if( pInfo->bListen ) {
				lEvent |= FD_ACCEPT;
			}
			if( WSAAsyncSelect( pInfo->hSocket, hWnd, pInfo->uAsyncMsg, lEvent ) != 0 ) {
				pInfo->uErrorID = IDS_ERROR_SELECT;
				ret = FALSE;
			}
		}
		if( !ret ) {
			pInfo->dwErrorCode = WSAGetLastError();
			if( pInfo->hSocket != INVALID_SOCKET ) {
				closesocket( pInfo->hSocket );
				pInfo->hSocket = INVALID_SOCKET;
			}
		}
	}
	if( ret ) {
		if( pInfo->hMsgBuff == NULL ) {
			pInfo->dwBuffLen = sizeof(MyClient2ServerMsg);
			pInfo->hMsgBuff = GlobalAlloc( GPTR, pInfo->dwBuffLen );
			pInfo->dwRecvLen = 0;
		}
	}
	return ret;
}

BOOL	CloseMySocket( HWND hWnd, NpMySocketInfo pInfo )
{
	BOOL	ret = FALSE;
	if( IsOpenedMySocket( pInfo ) ) {
		/*	NCAgؒf	*/
		CloseClientMySocket( hWnd, pInfo );
		
		shutdown( pInfo->hSocket, SD_BOTH );
		closesocket( pInfo->hSocket );
		pInfo->hSocket = INVALID_SOCKET;
		ret = TRUE;
	}
	return ret;
}

BOOL	IsOpenedMySocket( NpMySocketInfo pInfo )
{
	return( pInfo->hSocket != INVALID_SOCKET );
}

BOOL	IsExistClientMySocket( NpMySocketInfo pInfo )
{
	return( pInfo->hSockClient != INVALID_SOCKET );
}

BOOL	CloseClientMySocket( HWND hWnd, NpMySocketInfo pInfo )
{
	BOOL	ret = FALSE;
	if( pInfo->hSockClient != INVALID_SOCKET ) {
		shutdown( pInfo->hSockClient, SD_BOTH );
		closesocket( pInfo->hSockClient );
		pInfo->hSockClient = INVALID_SOCKET;
		ret = TRUE;
	}
	return ret;
}



void	MySockerMsgProc( HWND hWnd, WPARAM wParam, LPARAM lParam, NpMySocketInfo pInfo )
{
	SOCKET	hSocket = (SOCKET)wParam;
	
	if( WSAGETSELECTEVENT(lParam) == FD_CLOSE ) {
		if( pInfo->bListen ) {
#ifdef	_DEBUG
			OutputDebugString( _T("FD_CLOSE by peer\r\n") );
#endif
			/*	NCAgؒf	*/
			if( pInfo->hSockClient != INVALID_SOCKET && pInfo->hSockClient == hSocket ) {
				shutdown( pInfo->hSockClient, SD_BOTH );
				closesocket( pInfo->hSockClient );
				pInfo->hSockClient = INVALID_SOCKET;
				pInfo->dwRecvLen = 0;
#ifdef	_DEBUG
				OutputDebugString( _T("shutdown\r\n") );
#endif
				/*
					]Ãt@C폜BƂB
					낢냂[h߂B
					uf܂vbZ[W炢邩B
					󂯓mFoĂŒɁANCAg؂邱Ƃ邩B
				*/
				PostMessage( hWnd, WM_MY_SOCKET_NOTIFY, 0, 0 );
			}
		} else {
			/*	T[oؒf	*/
			CloseMySocket( hWnd, pInfo );
#ifdef	_DEBUG
			OutputDebugString( _T("FD_CLOSE myself\r\n") );
#endif
			PostMessage( hWnd, WM_MY_SOCKET_NOTIFY, 0, 0 );
		}
	} else if( WSAGETSELECTEVENT(lParam) == FD_READ ) {
		if( pInfo->bListen ) {
			/*	NCAg牽Ă	*/
			if( pInfo->dwRecvLen < pInfo->dwBuffLen ) {
				DWORD	dwLen = pInfo->dwBuffLen - pInfo->dwRecvLen;
				DWORD	dwRead = 0;
				LPBYTE	pt;
				pt = (LPBYTE)GlobalLock( pInfo->hMsgBuff );
				RecvMySocket( pInfo->hSockClient, pt + pInfo->dwRecvLen, dwLen, &dwRead );
#ifdef	_DEBUG
				{
					TCHAR	buff[100];
					wsprintf( buff, _T("Recv %ld, %ld, %ld\r\n"), pInfo->dwRecvLen, dwLen, dwRead );
					OutputDebugString( buff );
				}
#endif
				dwLen -= dwRead;
				pInfo->dwRecvLen += dwRead;
				if( dwLen == 0 ) {
					NpMyClient2ServerMsg	p = (NpMyClient2ServerMsg)pt;
					p->dwMagic = ntohl(p->dwMagic);
					p->wMsgType = ntohs(p->wMsgType);
					if( p->dwMagic == MSG_MAGIC ) {
#ifdef	_DEBUG
						TCHAR	buff[256];
						wsprintf( buff, _T("Recv message -- type:%d\r\n"), (int)(p->wMsgType) );
						OutputDebugString( buff );
#endif
						PostMessage( hWnd, WM_MY_SERVER_MSG, (WPARAM)(p->wMsgType), 0 );
						if( p->wMsgType == MSG_C2S_FILEINFO ) {
							p->uMsgBody.sFileInfo.dwSizeHigh = ntohl(p->uMsgBody.sFileInfo.dwSizeHigh);
							p->uMsgBody.sFileInfo.dwSizeLow = ntohl(p->uMsgBody.sFileInfo.dwSizeLow);
#ifdef	_DEBUG
							wsprintf( buff, _T("INFO:size %ld, %ld\r\n"), p->uMsgBody.sFileInfo.dwSizeLow, sizeof(MyClient2ServerMsg) );
							OutputDebugString( buff );
							OutputDebugString( p->uMsgBody.sFileInfo.szFileName );
#endif
						} else if( p->wMsgType == MSG_C2S_CONTENT ) {
							p->uMsgBody.sFileContent.wSize = ntohs(p->uMsgBody.sFileContent.wSize);
#ifdef	_DEBUG
							wsprintf( buff, _T("CONT:size %ld\r\n"), (int)ntohs(p->uMsgBody.sFileContent.wSize) );
							OutputDebugString( buff );
#endif
						} else if( p->wMsgType == MSG_C2S_FILEATTR ) {
							p->uMsgBody.sFileAttr.wYear = ntohs(p->uMsgBody.sFileAttr.wYear);
							p->uMsgBody.sFileAttr.wMonth = ntohs(p->uMsgBody.sFileAttr.wMonth);
							p->uMsgBody.sFileAttr.wDay = ntohs(p->uMsgBody.sFileAttr.wDay);
							p->uMsgBody.sFileAttr.wDayOfWeek = ntohs(p->uMsgBody.sFileAttr.wDayOfWeek);
							p->uMsgBody.sFileAttr.wHour = ntohs(p->uMsgBody.sFileAttr.wHour);
							p->uMsgBody.sFileAttr.wMinute = ntohs(p->uMsgBody.sFileAttr.wMinute);
							p->uMsgBody.sFileAttr.wSecond = ntohs(p->uMsgBody.sFileAttr.wSecond);
							p->uMsgBody.sFileAttr.wMilliseconds = ntohs(p->uMsgBody.sFileAttr.wMilliseconds);
#ifdef	_DEBUG
							wsprintf( buff, _T("ATTR:%04d/%02d/%02d(%d) %02d:%02d:%02d.%03d\r\n"), 
								(int)p->uMsgBody.sFileAttr.wYear,
								(int)p->uMsgBody.sFileAttr.wMonth,
								(int)p->uMsgBody.sFileAttr.wDay,
								(int)p->uMsgBody.sFileAttr.wDayOfWeek,
								(int)p->uMsgBody.sFileAttr.wHour,
								(int)p->uMsgBody.sFileAttr.wMinute,
								(int)p->uMsgBody.sFileAttr.wSecond,
								(int)p->uMsgBody.sFileAttr.wMilliseconds
							);
							OutputDebugString( buff );
							wsprintf( buff, _T("md5:%02x%02x%02x%02x%02x%02x%02x%02x%02x%02x%02x%02x%02x%02x%02x%02x\r\n"),
								(int)p->uMsgBody.sFileAttr.md5[0],
								(int)p->uMsgBody.sFileAttr.md5[1],
								(int)p->uMsgBody.sFileAttr.md5[2],
								(int)p->uMsgBody.sFileAttr.md5[3],
								(int)p->uMsgBody.sFileAttr.md5[4],
								(int)p->uMsgBody.sFileAttr.md5[5],
								(int)p->uMsgBody.sFileAttr.md5[6],
								(int)p->uMsgBody.sFileAttr.md5[7],
								(int)p->uMsgBody.sFileAttr.md5[8],
								(int)p->uMsgBody.sFileAttr.md5[9],
								(int)p->uMsgBody.sFileAttr.md5[10],
								(int)p->uMsgBody.sFileAttr.md5[11],
								(int)p->uMsgBody.sFileAttr.md5[12],
								(int)p->uMsgBody.sFileAttr.md5[13],
								(int)p->uMsgBody.sFileAttr.md5[14],
								(int)p->uMsgBody.sFileAttr.md5[15]
							);
							OutputDebugString( buff );
#endif
						}
					} else {
						pInfo->dwRecvLen = 0;
#ifdef	_DEBUG
						OutputDebugString( _T("FD_READ but bad magic\r\n") );
#endif
					}
				}
				GlobalUnlock( pInfo->hMsgBuff );
#ifdef	_DEBUG
			} else {
				OutputDebugString( _T("FD_READ but no space for read\r\n") );
#endif
			}
		} else {
			/*	T[oԎԂĂ	*/
			MyServer2ClientMsg	msg;
			if( RecvS2cMsg( pInfo->hSocket, &msg ) ) {
#ifdef	_DEBUG
				TCHAR	buff[100];
				wsprintf( buff, _T("RecvS2cMsg %ld %ld\r\n"), pInfo->nClientState, (int)msg.wAnswer );
				OutputDebugString( buff );
#endif
				PostMessage( hWnd, WM_MY_CLIENT_MSG, (WPARAM)pInfo->nClientState, (LPARAM)msg.wAnswer );
			}
		}
		/*
			Mp̃obt@́ApInfoɗpӂĂKvB
		*/
	} else if( WSAGETSELECTEVENT(lParam) == FD_ACCEPT ) {
		/*	NCAgڑĂ	*/
		if( pInfo->hSocket != INVALID_SOCKET && hSocket == pInfo->hSocket ) {
			SOCKADDR_IN	sockAddr;
			int		len = sizeof(sockAddr);
			SOCKET	hNewSocket = INVALID_SOCKET;
			ZeroMemory( &sockAddr, len );
			/*		*/
			hNewSocket = accept( pInfo->hSocket, (struct sockaddr *)&sockAddr, &len );
			if( hNewSocket == INVALID_SOCKET ) {
				/*	s	*/
				DWORD	dwErrCode = WSAGetLastError();
			} else if( pInfo->hSockClient != INVALID_SOCKET || IsPopupWindowShown( hWnd ) ) {
				/*	ɐڑĂ or |bvAbvEBhEoĂ	*/
				SendS2cMsg( hNewSocket, MSG_S2C_NAK );
				shutdown( hNewSocket, SD_BOTH );
				closesocket( hNewSocket );
#ifdef	_DEBUG
				OutputDebugString( _T("Denied\r\n") );
#endif
			} else {
				/*	OK	*/
				SendS2cMsg( hNewSocket, MSG_S2C_ACK );
				pInfo->hSockClient = hNewSocket;
				/*
					Xe[^XςB
					UI(L)KvȂςB
					_CAOɃbZ[W𓊂āAł点̂B
				*/
#ifdef	_DEBUG
				OutputDebugString( _T("Accepted\r\n") );
#endif
			}
		}
	}
}

BOOL	RecvMySocket( SOCKET hSocket, LPBYTE pBuff, DWORD dwSize, LPDWORD lpdwReadCnt )
{
	BOOL	ret = FALSE;
	if( hSocket != INVALID_SOCKET ) {
		DWORD	nBytes = 0;
		DWORD	dwRead = 0;
		if( ioctlsocket( hSocket, FIONREAD, &nBytes ) != SOCKET_ERROR ) {
			if( nBytes != 0 ) {
				ZeroMemory( pBuff, dwSize );
				dwRead = recv( hSocket, (LPSTR)pBuff, dwSize, 0 );
				ret = TRUE;
			}
		}
		if( lpdwReadCnt != NULL ) {
			*lpdwReadCnt = dwRead;
		}
	}
	return ret;
}

static	BOOL	RecvS2cMsg( SOCKET hSocket, NpMyServer2ClientMsg p )
{
	MyServer2ClientMsg	msg;
	DWORD	dwRead = 0;
	DWORD	dwSize = sizeof(msg);
	LPBYTE	q = (LPBYTE)(&msg);
	
	while( TRUE ) {
		if( RecvMySocket( hSocket, q, dwSize, &dwRead ) ) {
			if( dwSize <= dwRead ) {
				break;
			}
			q += dwRead;
			dwSize -= dwRead;
		}
	}
	p->dwMagic = ntohl(msg.dwMagic);
	p->wAnswer = ntohs(msg.wAnswer);
	
	return( p->dwMagic == MSG_MAGIC );
}

BOOL	SendMySocket( SOCKET hSocket, const LPBYTE pBuff, DWORD dwSize )
{
	BOOL	ret = FALSE;
	if( hSocket != INVALID_SOCKET ) {
		int		nLen = dwSize;
		int		nDone = 0;
		int		nSent;
		ret = TRUE;
		while( TRUE ) {
			nSent = send( hSocket, (LPSTR)pBuff+nDone, nLen, 0 );
			if( nSent == SOCKET_ERROR ) {
				ret = FALSE;
				break;
			}
			if( nSent >= nLen ) {
				break;
			}
			nLen  -= nSent;
			nDone += nSent;
		}
	}
	return ret;
}

BOOL	SendS2cMsg( SOCKET hSocket, WORD wAnswer )
{
	MyServer2ClientMsg	msg;
	msg.dwMagic = htonl(MSG_MAGIC);
	msg.wAnswer = htons(wAnswer);
#ifdef	_DEBUG
	OutputDebugString( ( wAnswer == MSG_S2C_ACK ) ? _T("SendS2C ACK\r\n") : _T("SendS2C NAK\r\n") );
#endif
	return( SendMySocket( hSocket, (LPBYTE)(&msg), sizeof(msg) ) );
}

BOOL	SendC2sMsg( SOCKET hSocket, NpMyClient2ServerMsg p, WORD type )
{
	p->dwMagic = htonl(MSG_MAGIC);
	p->wMsgType = htons(type);
	if( type == MSG_C2S_FILEINFO ) {
		p->uMsgBody.sFileInfo.dwSizeHigh = htonl(p->uMsgBody.sFileInfo.dwSizeHigh);
		p->uMsgBody.sFileInfo.dwSizeLow = htonl(p->uMsgBody.sFileInfo.dwSizeLow);
	} else if( type == MSG_C2S_CONTENT ) {
		p->uMsgBody.sFileContent.wSize = htons(p->uMsgBody.sFileContent.wSize);
	} else if( type == MSG_C2S_FILEATTR ) {
		p->uMsgBody.sFileAttr.wYear = htons(p->uMsgBody.sFileAttr.wYear);
		p->uMsgBody.sFileAttr.wMonth = htons(p->uMsgBody.sFileAttr.wMonth);
		p->uMsgBody.sFileAttr.wDay = htons(p->uMsgBody.sFileAttr.wDay);
		p->uMsgBody.sFileAttr.wDayOfWeek = htons(p->uMsgBody.sFileAttr.wDayOfWeek);
		p->uMsgBody.sFileAttr.wHour = htons(p->uMsgBody.sFileAttr.wHour);
		p->uMsgBody.sFileAttr.wMinute = htons(p->uMsgBody.sFileAttr.wMinute);
		p->uMsgBody.sFileAttr.wSecond = htons(p->uMsgBody.sFileAttr.wSecond);
		p->uMsgBody.sFileAttr.wMilliseconds = htons(p->uMsgBody.sFileAttr.wMilliseconds);
	}
#ifdef	_DEBUG
	{
		TCHAR	buff[100];
		wsprintf( buff, _T("SendC2s %d\r\n"), (int)type );
		OutputDebugString( buff );
	}
#endif
	return( SendMySocket( hSocket, (LPBYTE)p, sizeof(MyClient2ServerMsg) ) );
}


BOOL	GetPeerName( NpMySocketInfo pInfo, LPTSTR pBuff )
{
	BOOL	ret = FALSE;
	SOCKADDR_IN	sockAddr;
	int		len = sizeof(sockAddr);
	ZeroMemory( &sockAddr, sizeof(sockAddr) );
	if( getpeername( pInfo->hSockClient, (struct sockaddr *)&sockAddr, &len ) == 0 ) {
		LPCTSTR	q = inet_ntoa(sockAddr.sin_addr);
		lstrcpy( pBuff, q );
		ret = TRUE;
	}
	return( ret );
}
